#ifndef FileIncluded
#define FileIncluded

// copyright (c) 1992 by Paul Wheaton
// 1916 Brooks #205, Missoula, MT  59801
//
// voice phone:  (406)543-7543
// modem phone:  (406)543-1144 (2400N81)
//  CompuServe:  72707,207
//    Internet:  72707.207@CompuServe.com

/*

Although these routines lean on ANSI stdio functions, many of them work
somewhat differently.   "File" is for binary mode and "TextFile" is for
text mode. The constructor will open the file (create it if it doesn't
exist).  It is the applications programmers responsibility to make sure
that the filename given is valid and possible.  If the file cannot be
opened, the program will be stopped.

Binary files:

Reading and writing may be done to any byte location that will fit into a
Long.  Writing more than a byte beyond the EOF will simply generate garbage
characters from the EOF up to the point you are now writing.  Reading
beyond EOF will result in a garbage read and the Read function will return
False.

Text files:

After opening, the first read will be done from the beginning of the file
and the first write will be done to the end of the file.

Note!: if a special buffer size has been requested and *denied* (due to
lack of heap space), a beep will sound but file access will continue
with the default buffer size.

*/

#include <stdio.h>
#include <io.h>

#include <Misc.h>
#include <ByteVec.h>

#define FileExists(FileName) (!access(FileName,0))
  // returns True if the file exists

char CurDiskDrive();

long DiskSpace(const char Drive='.');  // '.' means default drive
  // the value returned is in "K" or "kilobytes" or number of 1024 byte blocks
  // this is in case this library is on a computer that can handle capacity
  // of more than 2 gigs that a "long" could represent.

long FileSize(const char* FileName);
  // the value returned is in bytes

#define WriteThing(A) Write(&A,sizeof(A))
#define ReadThing(A) Read(&A,sizeof(A))
  // use only on things where the size can be properly determined with
  // "sizeof".  Use only with "File" not "TextFile"

void DeleteFile(const char* FileName);
  // if the file is deletable, it's deleted

#define ReadAndWrite False
#define ReadOnly True

class LowLevelFile
  {
      FILE* FilePointer;
      Bool Open;
      char* GivenFileName;
      char* Buf; // for future use to change the size of the buffer
      friend class File;
      friend class TextFile;
      friend class RecFileRef;
      friend class RecFile;
    public:
      LowLevelFile(const char* FileName,const char* Mode, int BufSize);
      ~LowLevelFile() {Close();}
      void Close();  // you can close the file early if you want
      void Flush() {fflush(FilePointer);}
        // flush your write buffers out to disk
      Long CurPos() {return Long(ftell(FilePointer));}
        // current file position:  where you are about to write to or read from
      void Seek(Long Offset) {fseek(FilePointer,Offset,0);}
      Long Size();  //  the file size in bytes
  };

class File:public LowLevelFile
  {
    public:
      File(const char* FileName,Bool Mode=ReadAndWrite,int BufSize=BUFSIZ);
      Bool Read(void *Buffer,Long Size=1);
      Bool Read(ByteVector& BV,Long Size=1);
      void Write(const void *Buffer,Long Size=1);
  };

class RecFile:public LowLevelFile
  {
      int RecSize;
    public:
      RecFile(const char* FileName,int RecordSize,Bool Mode,int BufSize);
      Bool Read(void *Buffer);
      Bool Read(void *Buffer,int Quan);
      void Seek(Long RecNum);
      Long CurRec();  // record number
      Long Size();  // number of recs
      void Write(const void *Buffer);
      void Write(const void *Buffer,int Quan);
  };

#define CreateRecFileClass(ClassName,StructType)                        \
                                                                        \
class ClassName;                                                        \
                                                                        \
class ClassName ## Ref                                                  \
  {                                                                     \
      ClassName* F;                                                     \
      ClassName ## Ref(ClassName* XF){F=XF;}                            \
      friend class ClassName;                                           \
    public:                                                             \
      void operator=(const StructType& X);                              \
      friend void operator=(StructType& X,ClassName ## Ref R);          \
  };                                                                    \
                                                                        \
class ClassName:public RecFile                                          \
  {                                                                     \
    public:                                                             \
      ClassName(const char* FileName,Bool Mode=ReadAndWrite,            \
          int BufSize=BUFSIZ):                                          \
          (FileName,sizeof(StructType),Mode,BufSize){}                  \
      Bool Read(StructType& X){return RecFile::Read(&X);}               \
      Bool Read(StructType* X,int Q){return RecFile::Read(X,Q);}        \
      void Write(const StructType& X){RecFile::Write(&X);}              \
      void Write(const StructType* X,int Q){RecFile::Write(X,Q);}       \
      ClassName ## Ref operator[](Long RecNum)                          \
        {Seek(RecNum); return ClassName ## Ref(this);}                  \
  };                                                                    \
                                                                        \
inline void ClassName ## Ref::operator=(const StructType& X)            \
  {F->Write(X);}                                                        \
inline void operator=(StructType& X,ClassName ## Ref R)                 \
  {(R.F)->Read(X);}

class TextFile:public LowLevelFile
  {
      Bool Started;  // used to figure out whether we're starting off appending or reading
    public:
      TextFile(const char* FileName,Bool Mode=ReadAndWrite,int BufSize=BUFSIZ):
          LowLevelFile(FileName,((Mode==ReadOnly)?("r"):("r+")),BufSize)
          {Started=False;}
      void Seek(Long Offset) {Started=True; LowLevelFile::Seek(Offset);}
      Bool Read(char* S);
      void Write(const char* S);
      void WriteLine(const char* S="");
  };

void FileCopying(File& DestFile, File& SourceFile, Long Size);
  // will not modify file positions before writing

void CopyFile(const char* DestFile, const char* SourceFile);
  // source file must be created.  if dest file already exists,
  // it will be deleted

Word CRC(File& F,Long FSize);  // CRC of file starting at current pos
Word CRC(File& F);  // CRC of entire file


#endif

/*

Example of a file of records:

struct XType
  {
    Long a,b,c;
  };

CreateRecFileClass(XFile,XType);

main()
  {
    XType X;
    XFile F("X.BIN");
    F[0]=X;
    X=F[0];
  }

*/

