// SCRNGRPH.CPP : methods for drawing boxes on the screen in
// graphics mode. Note the same header file is used as for text
// mode, so you can simply link in a different object module to
// select text or graphics.
// By Bruce Eckel.  (c) The C Gazette

#include <fg.h>  // Zortech's "flash graphics" package
    // All variables and functions starting with "fg_" are described
    // in the Zortech function reference manual.
#include "scrnbox.hpp"

static int char_width;       // To be used in cursor movement
static int char_height;      // calculations.
static int screen_height;    // flash graphics 0,0 is a lower left
       // corner; this is used to adjust to "regular" coords.
       // "cursor" to keep track of where the next character goes:
static fg_coord_t x_cursor = 0;
static fg_coord_t y_cursor = 0;

// special class and static object to insure proper
// initialization and cleanup.
class __display_init {
public:
  __display_init() {
    fg_init();
    char_width = fg_box_width(fg.charbox);
    char_height = fg_box_height(fg.charbox);
    screen_height = fg_box_height(fg.displaybox);
  }
  ~__display_init() { fg_term(); }
} __display_initializer;

// Although in graphics mode, the position and size arguments
// still refer to text-mode lines and columns:

screen_box::screen_box(unsigned ypos, unsigned xpos,
                       unsigned bheight, unsigned bwidth) :
  x(xpos), y(ypos), height(bheight), width(bwidth) {
  // Set up the graphics box:
  fg_box_t BOX;
  BOX[FG_X1] = x * char_width;
  BOX[FG_X2] = (x + width) * char_width;
  BOX[FG_Y1] = screen_height - (y + height) * char_height -1;
  BOX[FG_Y2] = screen_height -1 - y * char_height;
  boxrep = new fg_coord_t[4];  // note we are assigning to a void *
       // Save the box values (cast necessary because of void *):
  ((fg_coord_t *)boxrep)[0] = BOX[FG_X1];
  ((fg_coord_t *)boxrep)[1] = BOX[FG_X2];
  ((fg_coord_t *)boxrep)[2] = BOX[FG_Y1];
  ((fg_coord_t *)boxrep)[3] = BOX[FG_Y2];
       // Save the screen underneath the box:
  oldscreen = (unsigned *)new fg_color_t[fg_box_area(BOX)];
  fg_readbox(BOX, (fg_color_t *)oldscreen);
  fg_fillbox(FG_BLACK, FG_MODE_SET, ~0, BOX);
  fg_drawbox(FG_WHITE, FG_MODE_SET, ~0, FG_LINE_SOLID, BOX, fg.displaybox);
  fg_flush();
}

screen_box::~screen_box() {
  fg_box_t BOX;
  // Restore the box values:
  BOX[FG_X1] = ((fg_coord_t *)boxrep)[0];
  BOX[FG_X2] = ((fg_coord_t *)boxrep)[1];
  BOX[FG_Y1] = ((fg_coord_t *)boxrep)[2];
  BOX[FG_Y2] = ((fg_coord_t *)boxrep)[3];
  // Restore the screen underneath the box:
  fg_writebox(BOX, (fg_color_t *)oldscreen);
  fg_flush();
  delete boxrep;
  delete oldscreen;
}

void screen_box::move(int line) {
  // Note the use of character heights and widths:
  if(line < height && line >= 0) {
    y_cursor = screen_height - (y + line + 1) * char_height;
    x_cursor = x * char_width + 1;
  }
}

void screen_box::put_c(char c) {
  fg_putc(FG_WHITE, FG_MODE_SET, ~0, FG_ROT0, x_cursor, y_cursor, c,
          fg.displaybox);
  // bump the cursor forward:
  x_cursor += char_width;
}

void screen_box::emphasize(char c) {
  // Fill the box where the character will reside:
  fg_box_t charbox;
  charbox[FG_X1] = x_cursor;
  charbox[FG_Y1] = y_cursor;
  charbox[FG_X2] = x_cursor + char_width -1;
  charbox[FG_Y2] = y_cursor + char_height -1;
  fg_fillbox(FG_WHITE, FG_MODE_SET, ~0, charbox);
  // Now XOR the character into the box, so it's reverse video:
  fg_putc(FG_HIGHLIGHT, FG_MODE_XOR, ~0, FG_ROT0, x_cursor, y_cursor, c,
          fg.displaybox);
  // bump the cursor forward:
  x_cursor += char_width;
}