/* GENSORT.C - a general-purpose sort utility.  Only character string sort
 *             keys are supported.
 *
 * Usage:
 *    GENSORT input-file output-file [flags]
 *
 * Input-file  - name of file to be read
 *
 * Output-file - name of result file to be written
 *
 * Flags:
 *    /Fnnnn - specifies that the records to be sorted are fixed-length
 *             records which do not have CR/LF terminators at the end
 *             of each record.  'nnnn' specifies the record length, which
 *             may range up to 65532.  If this flag is used the /O (sort 
 *             offset) flag must also be used.
 *
 *    /Onnnn - specifies the offset of the sort key within the record.
 *
 *    /Lnnnn - specifies the max length of the sort key.
 * 
 *    /I     - specifies that sorting should be case-insensitive
 *
 * Written by Bob Jarvis
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <limits.h>
#include <string.h>

FILE *infile, *outfile;

char *infname = NULL, *outfname = NULL;

int    fixed_reclen = 0;
size_t sort_offset  = 0;
int offset_flag_present = 0;
size_t sort_length  = UINT_MAX;
int    case_insensitive = 0;

size_t io_bufsize = 1024;
char  *io_buf = NULL;

int    nptrs = 4096;
char  **recptrs;

char *usage = 
   "GENSORT - a general-purpose sort utility.  Only 0-terminated character string\n"
   "          sort keys are supported.\n\n"
   "Usage:\n"
   "   GENSORT input-file output-file [flags]\n"
   "      Input-file  - name of file to be read\n"
   "      Output-file - name of result file to be written\n"
   "      Flags:\n"
   "         /Fnnnn - specifies that the records to be sorted are fixed-length\n"
   "                  records which do not have CR/LF terminators at the end\n"
   "                  of each record.  'nnnn' specifies the record length, which\n"
   "                  may range up to 65532.  If this flag is used the /O (sort\n"
   "                  offset) flag must also be used.\n\n"
   "         /Onnnn - specifies the offset of the sort key within the record.\n"
   "                  (First byte in record is byte 1)\n\n"
   "         /Lnnnn - specifies the max length of the sort key.\n\n"
   "         /I     - specifies that sorting should be case-insensitive";

void write_rec(int n)
   {
   if(fixed_reclen)
      fwrite(recptrs[n], fixed_reclen, 1, outfile);
   else
      fputs(recptrs[n], outfile);
   }

int cmpfunc(char **pp1, char **pp2)     /* Generalized comparison function */
   {
   char *p1, *p2;

   p1 = *pp1;
   p2 = *pp2;

   if(!case_insensitive)
      return(strncmp(p1+sort_offset, p2+sort_offset, sort_length));
   else
      return(strncmpl(p1+sort_offset, p2+sort_offset, sort_length));
   }

void read_rec(int n)
   {
   char *workptr;

   if(fixed_reclen)
      {
      recptrs[n] = malloc(fixed_reclen);
      if(recptrs[n] == NULL)
         {
         printf("Unable to allocate %d'th record storage (%d bytes)\n",
                n, fixed_reclen);
         exit(1);
         }

      fread(recptrs[n], fixed_reclen, 1, infile);
      if(feof(infile))
         free(recptrs[n]);
      }
   else     /* Lines must be CR/LF terminated */
      {
      fgets(io_buf, io_bufsize, infile);
      if(!feof(infile))
         {
         while(io_buf[strlen(io_buf)-1] != '\n')   /* buffer too small */
            {
            io_bufsize += io_bufsize;     /* so double it */
            workptr = realloc(io_buf, io_bufsize);
            if(workptr == NULL)
               {
               printf("Unable to resize I/O buffer to %d bytes\n", io_bufsize);
               exit(2);
               }

            io_buf = workptr;

            fgets(io_buf+strlen(io_buf), io_bufsize-strlen(io_buf), infile);
            }

         recptrs[n] = malloc(strlen(io_buf)+1);
         if(recptrs[n] == NULL)
            {
            printf("Unable to allocate %d'th record storage (%d bytes)\n",
                   n, fixed_reclen);
            exit(3);
            }

         strcpy(recptrs[n], io_buf);
         }
      }
   }

void process_args(int argc, char **argv)
   {
   int i;

   for(i = 1 ; i < argc ; ++i)
      {
      if(*argv[i] == '/')
         {
         switch(toupper(*(argv[i]+1)))
            {
            case 'F':
               fixed_reclen = atoi(argv[i]+2);
               break;

            case 'O':
               sort_offset  = atoi(argv[i]+2)-1; /* convert to 0-based offset */
               offset_flag_present = 1;
               break;

            case 'L':
               sort_length  = atoi(argv[i]+2);
               break;

            case 'I':
               case_insensitive = 1;
               break;

            default:
               printf("Unrecognized switch '%s'\n", argv[i]);
               exit(4);
            }
         }
      else
         {
         if(infname == NULL)
            infname = argv[i];
         else if(outfname == NULL)
            outfname = argv[i];
         else
            {
            printf("Unrecognized argument '%s'\n", argv[i]);
            exit(5);
            }
         }
      }

   if(infname == NULL || outfname == NULL)
      {
      printf(usage);
      exit(6);
      }

   if(fixed_reclen && !offset_flag_present)
      {
      printf("Offset must be specified with fixed record length\n");
      exit(7);
      }

   if(strcmpl(infname, outfname) == 0)
      {
      printf("Input and output files must have different names\n");
      exit(8);
      }
   }

main(int argc, char **argv)
   {
   int i, nrecs;
   char **workptr;

   process_args(argc, argv);

   printf("GENSORT starting...");

   /* Open files */

   if(!fixed_reclen)
      {
      infile = fopen(infname, "r");
      outfile = fopen(outfname, "w");
      }
   else
      {
      infile = fopen(infname, "rb");
      outfile = fopen(outfname, "wb");
      }

   if(infile == NULL)
      {
      printf("Unable to open input file '%s'\n", infname);

      if(outfile != NULL)
         fclose(outfile);

      exit(9);
      }

   if(outfile == NULL)
      {
      printf("Unable to open output file '%s'\n", outfname);

      if(infile != NULL)
         fclose(infile);

      exit(10);
      }

   /* Allocate I/O buffer if not fixed record length */

   if(!fixed_reclen)
      {
      io_buf = malloc(io_bufsize);
      if(io_buf == NULL)
         {
         printf("Unable to allocate I/O buffer of %d bytes\n", io_bufsize);
         exit(11);
         }
      }

   /* Allocate the array of pointers to records */

   recptrs = malloc(nptrs * sizeof(char *));
   if(recptrs == NULL)
      {
      printf("Unable to allocate record pointer storage (%d bytes)\n",
             nptrs * sizeof(char *));
      exit(12);
      }

   printf("reading...");

   i = 1;      /* initialize outside of loop to allow restart */

   read_rec(0);

   while(1)
      {
      for(; !feof(infile) && i < nptrs ; ++i)
         read_rec(i);

      if(feof(infile))     /* all records have been read */
         {
         i -= 1;
         break;
         }

      nptrs += nptrs;      /* still more records to read */

      if(nptrs > UINT_MAX / sizeof(char *))     /* too many records */
         {
         printf("Too many records to be sorted\n");
         exit(13);
         }

      workptr = realloc(recptrs, nptrs * sizeof(char *));
      if(workptr == NULL)
         {
         printf("Unable to expand record pointer array to %d elements\n",
                nptrs);
         exit(14);
         }

      recptrs = workptr;
      }

   nrecs = i;

   printf("%d records - sorting...", nrecs);

   qsort(recptrs, nrecs, sizeof(char *), cmpfunc);

   printf("writing...");

   /* Now write everything back out, close it up, and go home */

   for(i = 0 ; i < nrecs ; ++i)
      write_rec(i);

   fclose(infile);
   fclose(outfile);

   printf("done!\n");
   }

